/******************************************************************************
 *
 * 
 *
 * Copyright (C) 1997-2000 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#ifndef TRANSLATOR_CZ_H
#define TRANSLATOR_CZ_H

#include "translator.h"

// The translation from English to Czech by Vlastimil Havran.
// In the cases where are more translations possible I hope
// that I have selected the most reasonable text. If not, please,
// mail the comments and text proposals to
//
// havran@fel.cvut.cz, 1999/04/11
//
// The first version of Czech strings with diacritics was implemented by
// Petr Prikryl (prikrylp@skil.cz). Vlastimil Havran is still
// the maintainer. The translation was based on translator.h
// of Doxygen versison 1.1.4 (from scratch) and updated for 1.1.5
// (including important updates of the up-to-1.1.4 strings).
// 
// The Czech strings in this file were first written in Windows 1250
// encoding. The conditional compilation ensures or the neutral
// functionality of the macro DECODE, or calling the WinToISO()
// method for on-line encoding conversion. If you want to maintain
// the source in the ISO Latin-2, do convert the source, change the
// conditional definition of the DECODE macro using the method
// ISOToWin() -- for conversion of strings for the Windows version.
// The version which does not call the function is slightly faster.
//
// Notice: Only one of the following tables is necessary for
// the support as well as the only one conversion private method
// of the TranslatorCzech class. However, the prototype was created
// under WindowsNT thus using Windows 1250 encoding. The second
// table and the method are here to switch to the ISO Latin-2 
// smoothly. And also, there is not a big loose of space when both
// tables and methods are supported.

/*! Translation tables for characters >127. They should work, 
 * but I would not put my hand into fire for that.
 */
static const char WinToISOTab[] = {
    '\x80', '\x81', '\x82', '\x83', '\x84', '\x85', '\x86', '\x87',
    '\x88', '\x89', '\xA9', '\x8B', '\xA6', '\xAB', '\xAE', '\xAC',
    '\x90', '\x91', '\x92', '\x93', '\x94', '\x2E', '\x96', '\x97',
    '\x98', '\x99', '\xB9', '\x9B', '\xB6', '\xBB', '\xBE', '\xBC',
    '\xA0', '\x20', '\x20', '\xA3', '\xA4', '\xA1', '\xA6', '\xA7',
    '\x22', '\xA9', '\xAA', '\x3C', '\xAC', '\x2D', '\xAE', '\xAF',
    '\x2E', '\x2B', '\x20', '\xB3', '\x27', '\x75', '\xB6', '\xB7',
    '\x20', '\xB1', '\xBA', '\x3E', '\xA5', '\x22', '\xB5', '\xBF',
    '\xC0', '\xC1', '\xC2', '\xC3', '\xC4', '\xC5', '\xC6', '\xC7',
    '\xC8', '\xC9', '\xCA', '\xCB', '\xCC', '\xCD', '\xCE', '\xCF',
    '\xD0', '\xD1', '\xD2', '\xD3', '\xD4', '\xD5', '\xD6', '\xD7',
    '\xD8', '\xD9', '\xDA', '\xDB', '\xDC', '\xDD', '\xDE', '\xDF',
    '\xE0', '\xE1', '\xE2', '\xE3', '\xE4', '\xE5', '\xE6', '\xE7',
    '\xE8', '\xE9', '\xEA', '\xEB', '\xEC', '\xED', '\xEE', '\xEF',
    '\xF0', '\xF1', '\xF2', '\xF3', '\xF4', '\xF5', '\xF6', '\x2D',
    '\xF8', '\xF9', '\xFA', '\xFB', '\xFC', '\xFD', '\xFE', '\xFF',
    '\0'
};

static const char ISOToWinTab[] = {
    '\x80', '\x81', '\x82', '\x83', '\x84', '\x85', '\x86', '\x87',
    '\x88', '\x89', '\x8A', '\x8B', '\x8C', '\x8D', '\x8E', '\x8F',
    '\x90', '\x91', '\x92', '\x93', '\x94', '\x95', '\x96', '\x97',
    '\x98', '\x99', '\x9A', '\x9B', '\x9C', '\x9D', '\x9E', '\x9F',
    '\xA0', '\xA5', '\xA2', '\xA3', '\xA4', '\xBC', '\x8C', '\xA7',
    '\xA8', '\x8A', '\xAA', '\x8D', '\x8F', '\xAD', '\x8E', '\xAF',
    '\xB0', '\xB9', '\xB2', '\xB3', '\xB4', '\xBE', '\x9C', '\xB7',
    '\xB8', '\x9A', '\xBA', '\x9D', '\x9F', '\xBD', '\x9E', '\xBF',
    '\xC0', '\xC1', '\xC2', '\xC3', '\xC4', '\xC5', '\xC6', '\xC7',
    '\xC8', '\xC9', '\xCA', '\xCB', '\xCC', '\xCD', '\xCE', '\xCF',
    '\xD0', '\xD1', '\xD2', '\xD3', '\xD4', '\xD5', '\xD6', '\xD7',
    '\xD8', '\xD9', '\xDA', '\xDB', '\xDC', '\xDD', '\xDE', '\xDF',
    '\xE0', '\xE1', '\xE2', '\xE3', '\xE4', '\xE5', '\xE6', '\xE7',
    '\xE8', '\xE9', '\xEA', '\xEB', '\xEC', '\xED', '\xEE', '\xEF',
    '\xF0', '\xF1', '\xF2', '\xF3', '\xF4', '\xF5', '\xF6', '\xF7',
    '\xF8', '\xF9', '\xFA', '\xFB', '\xFC', '\xFD', '\xFE', '\xFF',
    '\0'
};


// Here the DECODE macro assumes the source written in Win1250 encoding.
//
#ifdef _WIN32
  #define DECODE(sInput) sInput
#else
  #define DECODE(sInput) WinToISO(sInput)
#endif



class TranslatorCzech : public Translator
{
  private:
    /*! returns the string converted from Win1250 to ISO Latin-2 */
    QCString WinToISO(const QCString sInput)
    {
      QCString result;
      int len = sInput.length();

      for (int i = 0; i < len; ++i)
      {
        unsigned int c = sInput[i];  
        result += (c > 127) ? WinToISOTab[c & 0x7F] : c;
      }
      return result;
    }
    
    /*! returns the string converted from ISO Latin-2 to Win1250 */
    QCString ISOToWin(const QCString sInput)
    {
      QCString result;
      int len = sInput.length();

      for (int i = 0; i < len; ++i)
      {
        unsigned int c = sInput[i];  
        result += (c > 127) ? ISOToWinTab[c & 0x7F] : c;
      }
      return result;
    }
    
  public:
    virtual QCString idLanguage()
    { return "czech"; }

    virtual QCString latexLanguageSupportCommand()
    { return "\\usepackage{czech}\n"; }
    
    /*! returns the name of the package that is included by LaTeX */
    virtual QCString latexBabelPackage() 
    { return ""; }

    /*! used in the compound documentation before a list of related functions. */
    virtual QCString trRelatedFunctions()
    { return DECODE("Souvisejc funkce"); }

    /*! subscript for the related functions. */
    virtual QCString trRelatedSubscript()
    { return DECODE("(Uveden funkce nejsou lenskmi funkcemi.)"); }

    /*! header that is put before the detailed description of files, classes and namespaces. */
    virtual QCString trDetailedDescription()
    { return DECODE("Detailn popis"); }

    /*! header that is put before the list of typedefs. */
    virtual QCString trMemberTypedefDocumentation()
    { return DECODE("Dokumentace k lenskch typm"); }
    
    /*! header that is put before the list of enumerations. */
    virtual QCString trMemberEnumerationDocumentation()
    { return DECODE("Dokumentace k lenskm vtm"); }
    
    /*! header that is put before the list of member functions. */
    virtual QCString trMemberFunctionDocumentation()
    { return DECODE("Dokumentace k metodm"); }
    
    /*! header that is put before the list of member attributes. */
    virtual QCString trMemberDataDocumentation()
    { return DECODE("Dokumentace k datovm lenm"); }

    /*! this is the text of a link put after brief descriptions. */
    virtual QCString trMore() 
    { return DECODE("(...)"); }

    /*! put in the class documentation */
    virtual QCString trListOfAllMembers()
    { return DECODE("Seznam vech len."); }

    /*! used as the title of the "list of all members" page of a class */
    virtual QCString trMemberList()
    { return DECODE("Seznam len tdy"); }

    /*! this is the first part of a sentence that is followed by a class name */
    virtual QCString trThisIsTheListOfAllMembers()
    { return DECODE("Zde naleznete pln seznam len tdy "); }

    /*! this is the remainder of the sentence after the class name */
    virtual QCString trIncludingInheritedMembers()
    { return DECODE(", vetn vech zddnch len."); }
    
    /*! this is put at the author sections at the bottom of man pages.
     *  parameter s is name of the project name.
     */
    virtual QCString trGeneratedAutomatically(const char *s)
    { QCString result="Generovno automaticky programem Doxygen "
                      "ze zdrojovch text";
      if (s) result+=(QCString)" projektu "+s;
      result+="."; 
      return DECODE(result);
    }

    /*! put after an enum name in the list of all members */
    virtual QCString trEnumName()
    { return DECODE("jmno vtu"); }
    
    /*! put after an enum value in the list of all members */
    virtual QCString trEnumValue()
    { return DECODE("hodnota vtu"); }
    
    /*! put after an undocumented member in the list of all members */
    virtual QCString trDefinedIn()
    { return DECODE("definovn v"); }

    /*! put as in introduction in the verbatim header file of a class.
     *  parameter f is the name of the include file.
     */
    virtual QCString trVerbatimText(const char *f)
    { return DECODE((QCString)"pln text vkldanho souboru "+f+"."); }
    
    // quick reference sections

    /*! This is put above each page as a link to the list of all groups of 
     *  compounds or files (see the \group command).
     */
    virtual QCString trModules()
    { return DECODE("Moduly"); }
    
    /*! This is put above each page as a link to the class hierarchy */
    virtual QCString trClassHierarchy()
    { return DECODE("Hierarchie td"); }
    
    /*! This is put above each page as a link to the list of annotated classes */
    virtual QCString trCompoundList()
    { return DECODE("Seznam td"); }
    
    /*! This is put above each page as a link to the list of documented files */
    virtual QCString trFileList()
    { return DECODE("Seznam soubor"); }

    /*! This is put above each page as a link to the list of all verbatim headers */
    virtual QCString trHeaderFiles()
    { return DECODE("Seznam hlavikovch soubor"); }

    /*! This is put above each page as a link to all members of compounds. */
    virtual QCString trCompoundMembers()
    { return DECODE("Seznam len td"); }

    /*! This is put above each page as a link to all members of files. */
    virtual QCString trFileMembers()
    { return DECODE("Symboly v souborech"); }

    /*! This is put above each page as a link to all related pages. */
    virtual QCString trRelatedPages()
    { return DECODE("Souvisejc strnky"); }

    /*! This is put above each page as a link to all examples. */
    virtual QCString trExamples()
    { return DECODE("Pklady"); }

    /*! This is put above each page as a link to the search engine. */
    virtual QCString trSearch()
    { return DECODE("Hledat"); }

    /*! This is an introduction to the class hierarchy. */
    virtual QCString trClassHierarchyDescription()
    { return DECODE("Zde naleznete seznam, vyjadujc vztah ddinosti td. "
             "Je seazen piblin (ale ne pln) podle abecedy:");
    }

    /*! This is an introduction to the list with all files. */
    virtual QCString trFileListDescription(bool extractAll)
    {
      QCString result="Zde naleznete seznam vech ";
      if (!extractAll) result+="dokumentovanch ";
      result+="soubor se strunmi popisy:";
      return DECODE(result);
    }

    /*! This is an introduction to the annotated compound list. */
    virtual QCString trCompoundListDescription()
    { return DECODE("Nsledujc seznam obsahuje pedevm identifikace td, "
             "ale nachz se zde i dal netriviln prvky, jako jsou "
             "struktury (struct), unie (union) a rozhran (interface). "
             "V seznamu jsou uvedeny jejich strun popisy:"); 
    }

    /*! This is an introduction to the page with all class members. */
    virtual QCString trCompoundMembersDescription(bool extractAll)
    {
      QCString result="Zde naleznete seznam vech ";
      if (!extractAll) result+="dokumentovanch ";
      result+="len td s odkazy na ";
      if (extractAll) 
        result+="dokumentaci td, ke kterm psluej:";
      else 
        result+="tdy, ke kterm psluej:";
      return DECODE(result);
    }

    /*! This is an introduction to the page with all file members. */
    virtual QCString trFileMembersDescription(bool extractAll)
    {
      QCString result="Zde naleznete seznam vech ";
      if (!extractAll) result+="dokumentovanch ";
      result+="symbol, kter jsou definovny na rovni svch soubor. "
              "Pro kad symbol je uveden odkaz na ";
      if (extractAll) 
        result+="dokumentaci pslunho souboru";
      else 
        result+="soubor, ve kterm je symbol definovn";
      result+=". Podle zpsobu definice me bt symbol globlnm symbolem "
              "nebo symbolem, kter je viditeln pouze z danho souboru "
              "(to se tk napklad statickch promnnch v C++). "
              "Seznam neobsahuje symboly len td. "
              "Symbol me oznaovat makro, typ, instanci tdy, "
              "promnnou, konstantu, funkci, vet, hodnotu vtu a podobn:";  
        
      return DECODE(result);
    }

    /*! This is an introduction to the page with the list of all header files. */
    virtual QCString trHeaderFilesDescription()
    { return DECODE("Zde naleznete hlavikov soubory, kter tvo "
             "aplikan programtorsk rozhran (API):"); }

    /*! This is an introduction to the page with the list of all examples */
    virtual QCString trExamplesDescription()
    { return DECODE("Zde naleznete seznam vech pklad:"); }

    /*! This is an introduction to the page with the list of related pages */
    virtual QCString trRelatedPagesDescription()
    { return DECODE("Zde naleznete seznam vech souvisejcch strnek "
                    "dokumentace:"); }

    /*! This is an introduction to the page with the list of class/file groups */
    virtual QCString trModulesDescription()
    { return DECODE("Zde naleznete seznam vech modul:"); }

    /*! This sentences is used in the annotated class/file lists if no brief
     * description is given. 
     */
    virtual QCString trNoDescriptionAvailable()
    { return DECODE("Popis nen k dispozici"); }
    
    // index titles (the project name is prepended for these) 


    /*! This is used in HTML as the title of index.html. */
    virtual QCString trDocumentation()
    { return DECODE("Dokumentace"); }

    /*! This is used in LaTeX as the title of the chapter with the 
     * index of all groups.
     */
    virtual QCString trModuleIndex()
    { return DECODE("Rejstk modul"); }

    /*! This is used in LaTeX as the title of the chapter with the 
     * class hierarchy.
     */
    virtual QCString trHierarchicalIndex()
    { return DECODE("Rejstk hierarchie td"); }

    /*! This is used in LaTeX as the title of the chapter with the 
     * annotated compound index.
     */
    virtual QCString trCompoundIndex()
    { return DECODE("Rejstk td"); }

    /*! This is used in LaTeX as the title of the chapter with the
     * list of all files.
     */
    virtual QCString trFileIndex() 
    { return DECODE("Rejstk soubor"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all groups.
     */
    virtual QCString trModuleDocumentation()
    { return DECODE("Dokumentace modul"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all classes, structs and unions.
     */
    virtual QCString trClassDocumentation()
    { return DECODE("Dokumentace td"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all files.
     */
    virtual QCString trFileDocumentation()
    { return DECODE("Dokumentace soubor"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all examples.
     */
    virtual QCString trExampleDocumentation()
    { return DECODE("Dokumentace pklad"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all related pages.
     */
    virtual QCString trPageDocumentation()
    { return DECODE("Dokumentace souvisejcch strnek"); }

    /*! This is used in LaTeX as the title of the document */
    virtual QCString trReferenceManual()
    { return DECODE("Referenn pruka"); }
    
    /*! This is used in the documentation of a file as a header before the 
     *  list of defines
     */
    virtual QCString trDefines()
    { return DECODE("Definice maker"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of function prototypes
     */
    virtual QCString trFuncProtos()
    { return DECODE("Prototypy"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of typedefs
     */
    virtual QCString trTypedefs()
    { return DECODE("Definice typ"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of enumerations
     */
    virtual QCString trEnumerations()
    { return DECODE("Vty"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of (global) functions
     */
    virtual QCString trFunctions()
    { return DECODE("Funkce"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of (global) variables
     */
    virtual QCString trVariables()
    { return DECODE("Promnn"); }

    /*! This is used in the documentation of a file as a header before the 
     *  list of (global) variables
     */
    virtual QCString trEnumerationValues()
    { return DECODE("Hodnoty vt"); }
    
    /*! This is used in man pages as the author section. */
    virtual QCString trAuthor()
    { return DECODE("Autor"); }

    /*! This is used in the documentation of a file before the list of
     *  documentation blocks for defines
     */
    virtual QCString trDefineDocumentation()
    { return DECODE("Dokumentace k definicm maker"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for function prototypes
     */
    virtual QCString trFunctionPrototypeDocumentation()
    { return DECODE("Dokumentace prototyp"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for typedefs
     */
    virtual QCString trTypedefDocumentation()
    { return DECODE("Dokumentace definic typ"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for enumeration types
     */
    virtual QCString trEnumerationTypeDocumentation()
    { return DECODE("Dokumentace vtovch typ"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for enumeration values
     */
    virtual QCString trEnumerationValueDocumentation()
    { return DECODE("Dokumentace vtovch hodnot"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for functions
     */
    virtual QCString trFunctionDocumentation()
    { return DECODE("Dokumentace funkc"); }

    /*! This is used in the documentation of a file/namespace before the list 
     *  of documentation blocks for variables
     */
    virtual QCString trVariableDocumentation()
    { return DECODE("Dokumentace promnnch"); }

    /*! This is used in the documentation of a file/namespace/group before 
     *  the list of links to documented compounds
     */
    virtual QCString trCompounds()
    { return DECODE("Tdy"); }

    /*! This is used in the documentation of a group before the list of 
     *  links to documented files
     */
    virtual QCString trFiles()
    { return DECODE("Soubory"); }

    /*! This is used in the standard footer of each page and indicates when 
     *  the page was generated 
     */
    virtual QCString trGeneratedAt(const char *date,const char *projName)
    { 
      QCString result=(QCString)"Generovno "+date;
      if (projName) result+=(QCString)" pro projekt "+projName;
      result+=(QCString)" programem";
      return DECODE(result);
    }
    /*! This is part of the sentence used in the standard footer of each page.
     */
    virtual QCString trWrittenBy()
    {
      return DECODE(" -- autor ");
    }

    /*! this text is put before a class diagram */
    virtual QCString trClassDiagram(const char *clName)
    {
      return DECODE((QCString)"Diagram ddinosti pro tdu "+clName);
    }
    
    /*! this text is generated when the \internal command is used. */
    virtual QCString trForInternalUseOnly()
    { return DECODE("Pouze pro vnitn pouit."); }

    /*! this text is generated when the \reimp command is used. */
    virtual QCString trReimplementedForInternalReasons()
    { return DECODE("Reimplementovno z internch dvod; "
                    "aplikan rozhran zachovno."); }

    /*! this text is generated when the \warning command is used. */
    virtual QCString trWarning()
    { return DECODE("Pozor"); }

    /*! this text is generated when the \bug command is used. */
    virtual QCString trBugsAndLimitations()
    { return DECODE("Chyby a omezen"); }

    /*! this text is generated when the \version command is used. */
    virtual QCString trVersion()
    { return DECODE("Verze"); }

    /*! this text is generated when the \date command is used. */
    virtual QCString trDate()
    { return DECODE("Datum"); }

    /*! this text is generated when the \author command is used. */
    virtual QCString trAuthors()
    { return DECODE("Autor/autoi"); }

    /*! this text is generated when the \return command is used. */
    virtual QCString trReturns()
    { return DECODE("Vrac"); }

    /*! this text is generated when the \sa command is used. */
    virtual QCString trSeeAlso()
    { return DECODE("Viz tak"); }

    /*! this text is generated when the \param command is used. */
    virtual QCString trParameters()
    { return DECODE("Parametry"); }

    /*! this text is generated when the \exception command is used. */
    virtual QCString trExceptions()
    { return DECODE("Vjimky"); }
    
    /*! this text is used in the title page of a LaTeX document. */
    virtual QCString trGeneratedBy()
    { return DECODE("Generovno programem"); }

    // new since 0.49-990307
    
    /*! used as the title of page containing all the index of all namespaces. */
    virtual QCString trNamespaceList()
    { return DECODE("Seznam prostor jmen"); }

    /*! used as an introduction to the namespace list */
    virtual QCString trNamespaceListDescription(bool extractAll)
    {
      QCString result="Zde naleznete seznam vech ";
      if (!extractAll) result+="dokumentovanch ";
      result+="prostor jmen se strunm popisem:";
      return DECODE(result);
    }

    /*! used in the class documentation as a header before the list of all
     *  friends of a class
     */
    virtual QCString trFriends()
    { return DECODE("Friends"); }
    
//////////////////////////////////////////////////////////////////////////
// new since 0.49-990405
//////////////////////////////////////////////////////////////////////////
    
    /*! used in the class documentation as a header before the list of all
     * related classes 
     */
    virtual QCString trRelatedFunctionDocumentation()
    { return DECODE("Dokumentace k friends"); }
    
//////////////////////////////////////////////////////////////////////////
// new since 0.49-990425
//////////////////////////////////////////////////////////////////////////

    /*! used as the title of the HTML page of a class/struct/union */
    virtual QCString trCompoundReference(const char *clName,
                                    ClassDef::CompoundType compType,
                                    bool isTemplate)
    {
      QCString result("Dokumentace ");
      if (isTemplate) result+="ablony ";
      switch(compType)
      {
        case ClassDef::Class:      result+="tdy "; break;
        case ClassDef::Struct:     result+="struktury "; break;
        case ClassDef::Union:      result+="unie "; break;
        case ClassDef::Interface:  result+="rozhran "; break;
        case ClassDef::Exception:  result+="vjimky "; break;
      }
      result+=clName;
      return DECODE(result);
    }

    /*! used as the title of the HTML page of a file */
    virtual QCString trFileReference(const char *fileName)
    {
      QCString result="Dokumentace souboru ";
      result+=fileName; 
      return DECODE(result);
    }

    /*! used as the title of the HTML page of a namespace */
    virtual QCString trNamespaceReference(const char *namespaceName)
    {
      QCString result("Dokumentace prostroru jmen ");
      result+=namespaceName;
      return DECODE(result);
    }
    
    /*! \mgroup Class sections
     *  these are for the member sections of a class, struct or union 
     */
    virtual QCString trPublicMembers()
    { return DECODE("Veejn metody"); }
    virtual QCString trPublicSlots()
    { return DECODE("Veejn sloty"); }
    virtual QCString trSignals()
    { return DECODE("Signly"); }
    virtual QCString trStaticPublicMembers()
    { return DECODE("Statick veejn metody"); }
    virtual QCString trProtectedMembers()
    { return DECODE("Chrnn metody"); }
    virtual QCString trProtectedSlots()
    { return DECODE("Chrnn sloty"); }
    virtual QCString trStaticProtectedMembers()
    { return DECODE("Statick chrnn metody"); }
    virtual QCString trPrivateMembers()
    { return DECODE("Privtn metody"); }
    virtual QCString trPrivateSlots()
    { return DECODE("Privtn sloty"); }
    virtual QCString trStaticPrivateMembers()
    { return DECODE("Statick privtn metody"); }
    /*! \endmgroup */ 
    
    /*! this function is used to produce a comma-separated list of items.
     *  use generateMarker(i) to indicate where item i should be put.
     */
    virtual QCString trWriteList(int numEntries)
    {
      QCString result;
      int i;
      // the inherits list contain `numEntries' classes
      for (i=0;i<numEntries;i++) 
      {
        // use generateMarker to generate placeholders for the class links!
        result+=generateMarker(i); // generate marker for entry i in the list 
                                   // (order is left to right)
        
        if (i!=numEntries-1)  // not the last entry, so we need a separator
        {
          if (i<numEntries-2) // not the fore last entry 
            result+=", ";
          else                // the fore last entry
            result+=" a ";
        }
      }
      return DECODE(result); 
    }
    
    /*! used in class documentation to produce a list of base classes,
     *  if class diagrams are disabled.
     */
    virtual QCString trInheritsList(int numEntries)
    {
      return DECODE("Dd "+trWriteList(numEntries)+".");
    }

    /*! used in class documentation to produce a list of super classes,
     *  if class diagrams are disabled.
     */
    virtual QCString trInheritedByList(int numEntries)
    {
      return DECODE("Zddna tdami "+trWriteList(numEntries)+".");
    }

    /*! used in member documentation blocks to produce a list of 
     *  members that are hidden by this one.
     */
    virtual QCString trReimplementedFromList(int numEntries)
    {
      return DECODE("Reimplementace metody tdy "+trWriteList(numEntries)+".");
    }

    /*! used in member documentation blocks to produce a list of
     *  all member that overwrite the implementation of this member.
     */
    virtual QCString trReimplementedInList(int numEntries)
    {
      return DECODE("Reimplementovno v "+trWriteList(numEntries)+".");
    }

    /*! This is put above each page as a link to all members of namespaces. */
    virtual QCString trNamespaceMembers()
    { return DECODE("Symboly v prostorech jmen"); }

    /*! This is an introduction to the page with all namespace members */
    virtual QCString trNamespaceMemberDescription(bool extractAll)
    { 
      QCString result="Zde naleznete seznam vech ";
      if (!extractAll) result+="dokumentovanch ";
      result+="symbol, kter jsou definovny ve svch prostorech jmen. "
              "U kadho je uveden odkaz na ";
      if (extractAll) 
        result+="dokumentaci pslunho prostoru jmen:";
      else 
        result+="pslun prostor jmen:";
      return DECODE(result);
    }
    /*! This is used in LaTeX as the title of the chapter with the 
     *  index of all namespaces.
     */
    virtual QCString trNamespaceIndex()
    { return DECODE("Rejstk prostor jmen"); }

    /*! This is used in LaTeX as the title of the chapter containing
     *  the documentation of all namespaces.
     */
    virtual QCString trNamespaceDocumentation()
    { return DECODE("Dokumentace prostor jmen"); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990522
//////////////////////////////////////////////////////////////////////////

    /*! This is used in the documentation before the list of all
     *  namespaces in a file.
     */
    virtual QCString trNamespaces()
    { return DECODE("Prostory jmen"); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990728
//////////////////////////////////////////////////////////////////////////

    /*! This is put at the bottom of a class documentation page and is
     *  followed by a list of files that were used to generate the page.
     */
    virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
        bool single)
    { // here s is one of " Class", " Struct" or " Union"
      // single is true implies a single file
      QCString result=(QCString)"Dokumentace pro ";
      switch(compType)
      {
        case ClassDef::Class:      result+="tuto tdu"; break;
        case ClassDef::Struct:     result+="tuto strukturu (struct)"; break;
        case ClassDef::Union:      result+="tuto unii (union)"; break;
        case ClassDef::Interface:  result+="toto rozhran"; break;
        case ClassDef::Exception:  result+="tuto vjimku"; break;
      }
      result+=" byla generovna z ";
      if (single) result+="nsledujcho souboru:"; 
      else        result+="nsledujcch soubor:";
      return DECODE(result);
    }

    /*! This is in the (quick) index as a link to the alphabetical compound
     * list.
     */
    virtual QCString trAlphabeticalList()
    { return DECODE("Rejstk td"); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990901
//////////////////////////////////////////////////////////////////////////

    /*! This is used as the heading text for the retval command. */
    virtual QCString trReturnValues()
    { return DECODE("Vracen hodnoty"); }

    /*! This is in the (quick) index as a link to the main page (index.html)
     */
    virtual QCString trMainPage()
    { return DECODE("Hlavn strnka"); }

    /*! This is used in references to page that are put in the LaTeX 
     *  documentation. It should be an abbreviation of the word page.
     */
    virtual QCString trPageAbbreviation()
    { return DECODE("s."); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-991003
//////////////////////////////////////////////////////////////////////////

    virtual QCString trSources()
    {
      return DECODE("Zdroje");
    }
    virtual QCString trDefinedAtLineInSourceFile()
    {
      return DECODE("Definice je uvedena na dku @0 v souboru @1.");
    }
    virtual QCString trDefinedInSourceFile()
    {
      return DECODE("Definice v souboru @0.");
    }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-991205
//////////////////////////////////////////////////////////////////////////

    virtual QCString trDeprecated()
    {
      return DECODE("Zastaral");
    }

//////////////////////////////////////////////////////////////////////////
// new since 1.0.0
//////////////////////////////////////////////////////////////////////////

    /*! this text is put before a collaboration diagram */
    virtual QCString trCollaborationDiagram(const char *clName)
    {
      return DECODE((QCString)"Diagram td pro "+clName+":");
    }
    /*! this text is put before an include dependency graph */
    virtual QCString trInclDepGraph(const char *fName)
    {
      return DECODE((QCString)"Graf zvislost na vkldanch souborech "
                    "pro "+fName+":");
    }
    /*! header that is put before the list of constructor/destructors. */
    virtual QCString trConstructorDocumentation()
    {
      return DECODE("Dokumentace konstruktoru a destruktoru"); 
    }
    /*! Used in the file documentation to point to the corresponding sources. */
    virtual QCString trGotoSourceCode()
    {
      return DECODE("Zobrazit zdrojov text tohoto souboru.");
    }
    /*! Used in the file sources to point to the corresponding documentation. */
    virtual QCString trGotoDocumentation()
    {
      return DECODE("Zobrazit dokumentaci tohoto souboru.");
    }
    /*! Text for the \pre command */
    virtual QCString trPrecondition()
    {
      return DECODE("Precondition");
    }
    /*! Text for the \post command */
    virtual QCString trPostcondition()
    {
      return DECODE("Postcondition");
    }
    /*! Text for the \invariant command */
    virtual QCString trInvariant()
    {
      return DECODE("Invariant");
    }
    /*! Text shown before a multi-line variable/enum initialization */
    virtual QCString trInitialValue()
    {
      return DECODE("Initializer:");
    }
    /*! Text used the source code in the file index */
    virtual QCString trCode()
    {
      return DECODE("zdrojov text");
    }
    virtual QCString trGraphicalHierarchy()
    {
      return DECODE("Grafick zobrazen hierarchie td");
    }
    virtual QCString trGotoGraphicalHierarchy()
    {
      return DECODE("Zobrazit grafickou podobu hierarchie td");
    }
    virtual QCString trGotoTextualHierarchy()
    {
      return DECODE("Zobrazit textovou podobu hierarchie td");
    }
    virtual QCString trPageIndex()
    {
      return DECODE("Rejstk strnek");
    }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.0
//////////////////////////////////////////////////////////////////////////
    
    virtual QCString trNote()
    {
      return DECODE("Poznmka");
    }
    virtual QCString trPublicTypes()
    {
      return DECODE("Veejn typy");
    }
    virtual QCString trPublicAttribs()
    {
      return DECODE("Veejn atributy");
    }
    virtual QCString trStaticPublicAttribs()
    {
      return DECODE("Statick veejn atributy");
    }
    virtual QCString trProtectedTypes()
    {
      return DECODE("Chrnn typy");
    }
    virtual QCString trProtectedAttribs()
    {
      return DECODE("Chrnn atributy");
    }
    virtual QCString trStaticProtectedAttribs()
    {
      return DECODE("Statick chrnn atributy");
    }
    virtual QCString trPrivateTypes()
    {
      return DECODE("Privtn typy");
    }
    virtual QCString trPrivateAttribs()
    {
      return DECODE("Privtn atributy");
    }
    virtual QCString trStaticPrivateAttribs()
    {
      return DECODE("Statick privtn atributy");
    }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.3
//////////////////////////////////////////////////////////////////////////

    /*! Used as a marker that is put before a todo item */
    virtual QCString trTodo()
    {
      return DECODE("Udlat");
    }
    /*! Used as the header of the todo list */
    virtual QCString trTodoList()
    {
      return DECODE("Seznam plnovanch prav");
    }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.4
//////////////////////////////////////////////////////////////////////////

    virtual QCString trReferencedBy()
    {
      return DECODE("Pouv se v");
    }
    virtual QCString trRemarks()
    {
      return DECODE("Poznmky"); // ??? not checked in a context
    }
    virtual QCString trAttention()
    {
      return DECODE("Upozornn"); // ??? not checked in a context
    }
    virtual QCString trInclByDepGraph()
    {
      return DECODE("Nsledujc graf ukazuje, kter soubory pmo nebo "
             "nepmo vkldaj tento soubor:");
    }
    virtual QCString trSince()
    {
      return DECODE("Od"); // ??? not checked in a context
    }
};

#endif // TRANSLATOR_CZ_H
